/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.cxf.interceptor;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.cxf.attachment.AttachmentDeserializer;
import org.apache.cxf.binding.soap.interceptor.ProviderReadHeadersInterceptor;
import org.apache.cxf.common.logging.LogUtils;
import org.apache.cxf.helpers.CastUtils;
import org.apache.cxf.helpers.HttpHeaderHelper;
import org.apache.cxf.message.Message;
import org.apache.cxf.phase.AbstractPhaseInterceptor;
import org.apache.cxf.phase.Phase;
import org.dom4j.io.SAXReader;

import com.sun.mail.iap.ByteArray;
 
public class ConsumerAttachmentInInterceptor extends AbstractPhaseInterceptor<Message> {

    private static final Logger LOG = LogUtils.getL7dLogger(ConsumerAttachmentInInterceptor.class);
    protected Log logger = LogFactory.getLog(ConsumerAttachmentInInterceptor.class);
    /**
     * contruct the soap message with attachments from mime input stream
     * 
     * @param messageParam
     */
    
    public ConsumerAttachmentInInterceptor() {
        super(Phase.RECEIVE);
    }
    
    public void handleMessage(Message message) {
    	
    	
    	
        
        
    	
         
        if (isGET(message)) {
            LOG.info("ConsumerAttachmentInInterceptor skipped in HTTP GET method");
            return;
        }
        
        String contentType = (String) message.get(Message.CONTENT_TYPE);
        
        
        if (contentType != null && contentType.toLowerCase().indexOf("multipart/related") != -1) {
        	logger.debug("== ConsumerAttachmentInInterceptor Request is Multipart ==");
        	AttachmentDeserializer ad = new AttachmentDeserializer(message);
            try {
                ad.initializeAttachments();
            } catch (IOException e) {
                throw new Fault(e);
            }
        }else if (isGZIPMessage(message)) {
        	logger.debug("==  ConsumerAttachmentInInterceptor :: Request is GZIPMessage == ");
        }else{
        	logger.debug("==  ConsumerAttachmentInInterceptor :: Request is NOT Multipart == ");
        	try{
            	InputStream in = (InputStream)message.getContent(InputStream.class);
            	SAXReader saxReader = new SAXReader();
            	org.dom4j.Document messageDOM = saxReader.read(in);
            	logger.debug("== ConsumerAttachmentInInterceptor ==");
            	String messageXML = messageDOM.asXML();
            	message.setContent(InputStream.class, new ByteArrayInputStream(messageXML.getBytes()));
            	logger.debug(messageXML);
            	logger.debug("=====================================");
            }catch (Exception e) {
    			e.printStackTrace();
    		}
        }
        
       
       
    }

    protected boolean isGZIPMessage(Message message){
   	 // check for Content-Encoding header - we are only interested in
       // messages that say they are gzipped.
       Map<String, List<String>> protocolHeaders = CastUtils.cast((Map<?, ?>)message
           .get(Message.PROTOCOL_HEADERS));
       if (protocolHeaders != null) {
           List<String> contentEncoding = CastUtils.cast(HttpHeaderHelper
               .getHeader(protocolHeaders, HttpHeaderHelper.CONTENT_ENCODING));
           if (contentEncoding != null && (contentEncoding.contains("gzip") || contentEncoding.contains("x-gzip"))) {
           	return true;
           }
       }
       return false;
   }
  
    public void handleFault(Message messageParam) {
    }

}

