/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.servicemix.cxfbc.interceptors;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamReader;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.cxf.binding.soap.SoapMessage;
import org.apache.cxf.binding.soap.SoapVersion;
import org.apache.cxf.binding.soap.model.SoapBindingInfo;
import org.apache.cxf.binding.soap.model.SoapBodyInfo;
import org.apache.cxf.endpoint.Endpoint;
import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.interceptor.URIMappingInterceptor;
import org.apache.cxf.message.Message;
import org.apache.cxf.phase.AbstractPhaseInterceptor;
import org.apache.cxf.phase.Phase;
import org.apache.cxf.phase.PhaseInterceptorChain;
import org.apache.cxf.service.model.BindingMessageInfo;
import org.apache.cxf.service.model.BindingOperationInfo;
import org.apache.cxf.service.model.OperationInfo;
import org.apache.cxf.service.model.ServiceModelUtil;
import org.apache.cxf.staxutils.DepthXMLStreamReader;
import org.apache.cxf.staxutils.StaxUtils;
import org.apache.servicemix.soap.util.DomUtil;
import org.dom4j.XPath;
import org.dom4j.io.SAXReader;
import org.dom4j.xpath.DefaultXPath;
import org.w3c.dom.Document;
import org.w3c.dom.Element;


public class JbiOperationInterceptor extends AbstractPhaseInterceptor<Message> {

	protected Log logger = LogFactory.getLog(JbiOperationInterceptor.class);
    public JbiOperationInterceptor() {
        super(Phase.UNMARSHAL);
        addAfter(URIMappingInterceptor.class.getName());
    }

    public void handleMessage(Message message) {
    	
        if (isGET(message)) {
        	logger.debug("JbiOperationInterceptor -> isGET(message) true -> return");
            return;
        }
        if (message.getExchange().get(BindingOperationInfo.class) != null) {
        	logger.debug("JbiOperationInterceptor -> message.getExchange().get(BindingOperationInfo.class) != null return");
        	return;
        }
        
        Element body = getBodyElement((SoapMessage)message);
        BindingOperationInfo operation = null;
        Element el = DomUtil.getFirstChildElement(body);
        String opName = el.getLocalName();
        logger.debug("JbiOperationInterceptor -> OperationName at debug point 1 -> ["+opName+"]");
        logger.debug("----- isRequestor(Message) ["+isRequestor(message)+"]");
        if (isRequestor(message) && opName.endsWith("Response")) {
            opName = opName.substring(0, opName.length() - 8);
        }
        QName opQName = new QName(el.getNamespaceURI(), opName);
        SoapBindingInfo binding = (SoapBindingInfo) message.getExchange().get(
                Endpoint.class).getEndpointInfo().getBinding();
        
        for (BindingOperationInfo op : binding.getOperations()) {
        	
            String style = binding.getStyle(op.getOperationInfo());
            if (style == null) {
                style = binding.getStyle();
            }
            logger.debug(" -> Operation Style ["+style+"]");
            if ("document".equals(style)) {
                BindingMessageInfo msg = !isRequestor(message) ? op.getInput() : op.getOutput();
                
                
                logger.debug("---> msg.getExtensor(SoapBodyInfo.class).getParts().get(0).getElementQName() -> ["+msg.getExtensor(SoapBodyInfo.class).getParts().get(0).getElementQName()+"]");
                logger.debug("---> opQName ["+opQName+"]");
                if (msg.getExtensor(SoapBodyInfo.class).getParts().get(0).getElementQName().equals(opQName)) {
                    operation = op;
                    break;
                }
            } else {
                if (opQName.equals(op.getName())) {
                    operation = op;
                    break;
                }
            }
        }
        logger.debug("Operation ["+operation+"]");
        if (operation != null) {
        	logger.debug("Operation is NOT NULL ");
        	message.getExchange().put(BindingOperationInfo.class, operation);
            message.getExchange().put(OperationInfo.class,
                    operation.getOperationInfo());
            message.getExchange().setOneWay(operation.getOperationInfo().isOneWay());
        }else{
        	logger.debug("Operation is NULL ");
        }
       
        
        /*
        //DepthXMLStreamReader xmlReader = getXMLStreamReader(message);
        DepthXMLStreamReader xmlReader = null;
        XMLStreamReader xr = StaxUtils.createXMLStreamReader(new ByteArrayInputStream(messageXML.getBytes()));
        if (xr instanceof DepthXMLStreamReader) {
            xmlReader = (DepthXMLStreamReader) xr;
        }
        
        xmlReader  = new DepthXMLStreamReader(xr);
        if ( xmlReader == null ){
        	logger.debug("JbiOperationInterceptor XML Reader IS NULL");
        }
        BindingOperationInfo operation = null;
        if (!StaxUtils.toNextElement(xmlReader)) {
            message.setContent(Exception.class, new RuntimeException(
                    "There must be a method name element."));
        }
        String opName = xmlReader.getLocalName();
        logger.debug("JbiOperationInterceptor -> OperationName at debug point 1 -> ["+opName+"]");
        logger.debug("----- isRequestor(Message) ["+isRequestor(message)+"]");
        if (isRequestor(message) && opName.endsWith("Response")) {
            opName = opName.substring(0, opName.length() - 8);
        }
        logger.debug("JbiOperationInterceptor -> OperationName at debug point 2 -> ["+opName+"]");
        QName opQName = new QName(xmlReader.getNamespaceURI(), opName);
        SoapBindingInfo binding = (SoapBindingInfo) message.getExchange().get(
                Endpoint.class).getEndpointInfo().getBinding();
        
        logger.debug("JbiOperationInterceptor -> Obtained SoapBindingInfo ");
        logger.debug("JbiOperationInterceptor -> Looping all Binding Operation Info on SoapBinding ");
        
        for (BindingOperationInfo op : binding.getOperations()) {
        	
            String style = binding.getStyle(op.getOperationInfo());
            if (style == null) {
                style = binding.getStyle();
            }
            logger.debug(" -> Operation Style ["+style+"]");
            if ("document".equals(style)) {
                BindingMessageInfo msg = !isRequestor(message) ? op.getInput() : op.getOutput();
                
                
                logger.debug("---> msg.getExtensor(SoapBodyInfo.class).getParts().get(0).getElementQName() -> ["+msg.getExtensor(SoapBodyInfo.class).getParts().get(0).getElementQName()+"]");
                logger.debug("---> opQName ["+opQName+"]");
                if (msg.getExtensor(SoapBodyInfo.class).getParts().get(0).getElementQName().equals(opQName)) {
                    operation = op;
                    break;
                }
            } else {
                if (opQName.equals(op.getName())) {
                    operation = op;
                    break;
                }
            }
        }
        logger.debug("Operation ["+operation+"]");
        if (operation != null) {
        	logger.debug("Operation is NOT NULL ");
        	message.getExchange().put(BindingOperationInfo.class, operation);
            message.getExchange().put(OperationInfo.class,
                    operation.getOperationInfo());
            message.getExchange().setOneWay(operation.getOperationInfo().isOneWay());
        }else{
        	logger.debug("Operation is NULL ");
        }
        message.setContent(InputStream.class, new ByteArrayInputStream(messageXML.getBytes()));
        */
    }

    protected DepthXMLStreamReader getXMLStreamReader(Message message) {
    	
    
        XMLStreamReader xr = message.getContent(XMLStreamReader.class);
        if (xr instanceof DepthXMLStreamReader) {
            return (DepthXMLStreamReader) xr;
        }
        DepthXMLStreamReader dr = new DepthXMLStreamReader(xr);
        message.setContent(XMLStreamReader.class, dr);
        return dr;
    
    }

    protected BindingOperationInfo getOperation(Message message, QName opName) {
        BindingOperationInfo op = ServiceModelUtil.getOperation(message
                .getExchange(), opName);
        if (op == null) {
            throw new Fault(new Exception("Unrecognized operation"));
        }
        return op;
    }

    protected boolean isRequestor(Message message) {
        return Boolean.TRUE.equals(message.get(Message.REQUESTOR_ROLE));
    }

    /**
     * Extract the content as DOM element
     */
    protected Element getBodyElement(SoapMessage message) {
        try {
            InputStream in = (InputStream)message.getContent(InputStream.class);
            SAXReader saxReader = new SAXReader();
            org.dom4j.Document messageDOM = saxReader.read(in);
            logger.debug("== JbiOperationInInterceptor ==");
            String messageXML = messageDOM.asXML();
            message.setContent(InputStream.class, new ByteArrayInputStream(messageXML.getBytes()));
            logger.debug(messageXML);
            logger.debug("===============================");
          
            SoapVersion soapVersion = message.getVersion();
            
            Map<String, String> namespaceURI = new HashMap<String, String>();
            logger.debug(" soapVersion.getNamespace()) -> "+ soapVersion.getNamespace());
            namespaceURI.put("soap", soapVersion.getNamespace());
            
            XPath xpath = new DefaultXPath("/soap:Envelope/soap:Body");
            xpath.setNamespaceURIs(namespaceURI);
            org.dom4j.Node node = xpath.selectSingleNode(messageDOM);
            
            logger.debug("== Body is ==");
            logger.debug(node.asXML());
            
            Document domDocument = DomUtil.parse(new ByteArrayInputStream(node.asXML().getBytes()));
            return domDocument.getDocumentElement();
        } catch (Exception e) {
            throw new Fault(e);
        }
    }
}
